/*

	Framework - Window.cpp
	File Version 1.0.000

*/

#include "win32/Window.h"

using namespace Framework;

#define PROPNAME		_X("CWindow::this")

ATOM			CWindow::m_nAtom;

CWindow::CWindow()
{
	m_nNoCallDef = 0;
	m_hWnd = 0;
}

CWindow::~CWindow()
{

}

void CWindow::StdMsgLoop(CWindow* pW)
{
	MSG m;
	while(pW->IsWindow())
	{
		GetMessage(&m, NULL, NULL, NULL);
		TranslateMessage(&m);
		DispatchMessage(&m);
	}
}

void CWindow::SetClassPtr()
{
	SetProp(m_hWnd, PROPNAME, this);
}

void CWindow::ClearClassPtr()
{
	SetProp(m_hWnd, PROPNAME, NULL);
}

void CWindow::Initialize()
{
	m_nAtom = GlobalAddAtom(PROPNAME);
}

void CWindow::Release()
{
	GlobalDeleteAtom(m_nAtom);
}

unsigned int CWindow::DoesWindowClassExist(xchar* sClass)
{
	WNDCLASS wc;
	return GetClassInfo(GetModuleHandle(NULL), sClass, &wc);
}

///////////////////////////////////////////////////////////
//Window Message Helpers
///////////////////////////////////////////////////////////

void CWindow::Create(unsigned long nStyleEx, xchar* sClass, xchar* sWindow, unsigned long nStyle, RECT* pR, HWND hParent, void* pParam)
{
	m_hWnd = CreateWindowEx(nStyleEx, sClass, sWindow, nStyle, pR->left, pR->top, (pR->right - pR->left), (pR->bottom - pR->top), hParent, NULL, GetModuleHandle(NULL), pParam);
}

bool CWindow::IsWindow()
{
	return (::IsWindow(m_hWnd) ? true : false);
}

bool CWindow::IsVisible()
{
	long nStyle;
	nStyle = GetWindowLong(m_hWnd, GWL_STYLE);
	if(nStyle & WS_VISIBLE) return true;
	return false;
}

unsigned int CWindow::Destroy()
{
	return DestroyWindow(m_hWnd);
}

unsigned int CWindow::Show(int nMode)
{
	return ShowWindow(m_hWnd, nMode);
}

unsigned int CWindow::Enable(unsigned int nEnable)
{
	return EnableWindow(m_hWnd, nEnable);
}

int CWindow::GetText(xchar* sText, int nMax)
{
	return GetWindowText(m_hWnd, sText, nMax);
}

unsigned int CWindow::GetTextLength()
{
	return (unsigned int)SendMessage(m_hWnd, WM_GETTEXTLENGTH, 0, 0);
}

unsigned int CWindow::SetText(xchar* sText)
{
	return SetWindowText(m_hWnd, sText);
}

unsigned int CWindow::SetTextA(char* sText)
{
	return SetWindowTextA(m_hWnd, sText);
}

void CWindow::SetFocus()
{
	::SetFocus(m_hWnd);
}

void CWindow::SetFont(HFONT nFont)
{
	SendMessage(m_hWnd, WM_SETFONT, (WPARAM)nFont, TRUE);
}

void CWindow::SetMenu(HMENU nMenu)
{
	::SetMenu(m_hWnd, nMenu);
}

void CWindow::SetIcon(unsigned int nWhich, HICON nIcon)
{
	SendMessage(m_hWnd, WM_SETICON, nWhich, (LPARAM)nIcon);
}

HWND CWindow::GetParent()
{
	return ::GetParent(m_hWnd);
}

void CWindow::GetClientRect(RECT* pR)
{
	::GetClientRect(m_hWnd, pR);
}

void CWindow::Center(HWND hParent)
{
	RECT rParent;
	RECT rWindow;

	GetWindowRect(m_hWnd, &rWindow);
	if(hParent == NULL)
	{
		SetRect(&rParent, 0, 0, GetDeviceCaps(GetDC(NULL), HORZRES), GetDeviceCaps(GetDC(NULL), VERTRES));
	}
	else
	{
		::GetClientRect(hParent, &rParent);
	}

	SetWindowPos(m_hWnd, NULL, (rParent.right - (rWindow.right - rWindow.left)) / 2, (rParent.bottom - (rWindow.bottom - rWindow.top)) / 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);

}

void CWindow::SetSize(unsigned int nWidth, unsigned int nHeight)
{
	SetWindowPos(m_hWnd, NULL, 0, 0, nWidth, nHeight, SWP_NOMOVE | SWP_NOZORDER);
}

void CWindow::SetPosition(unsigned int nX, unsigned int nY)
{
	SetWindowPos(m_hWnd, NULL, nX, nY, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

void CWindow::Redraw()
{
	RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_UPDATENOW | RDW_INVALIDATE);
}

void CWindow::ModifyStyleOr(unsigned long nStyleBit)
{
	SetWindowLong(m_hWnd, GWL_STYLE, GetWindowLong(m_hWnd, GWL_STYLE) | nStyleBit);
}

///////////////////////////////////////////////////////////
//Window Message Handlers
///////////////////////////////////////////////////////////

long WINAPI CWindow::WndProc(HWND hWnd, unsigned int uiMsg, WPARAM wParam, LPARAM lParam)
{
	CWindow* pThis;
	pThis = (CWindow*)GetProp(hWnd, PROPNAME);
	if(pThis == NULL)
	{
		return (long)DefWindowProc(hWnd, uiMsg, wParam, lParam);
	}
	switch(uiMsg)
	{
	case WM_SYSCOMMAND:
		if(!pThis->OnSysCommand((unsigned int)wParam, lParam)) return FALSE;
		break;
	case WM_SIZE:
		if(!pThis->OnSize((unsigned int)wParam, LOWORD(lParam), HIWORD(lParam))) return FALSE;
		break;
	case WM_SETFOCUS:
		pThis->OnSetFocus();
		break;
	case WM_KILLFOCUS:
		pThis->OnKillFocus();
		break;
	case WM_PAINT:
		if(!pThis->OnPaint()) return FALSE;
		break;
	case WM_ERASEBKGND:
		if(!pThis->OnEraseBkgnd()) return FALSE;
		break;
	case WM_KEYDOWN:
		if(!pThis->OnKeyDown((unsigned int)wParam)) return FALSE;
		break;
	case WM_KEYUP:
		if(!pThis->OnKeyUp((unsigned int)wParam)) return FALSE;
		break;
	case WM_SYSKEYDOWN:
		if(!pThis->OnSysKeyDown((unsigned int)wParam)) return FALSE;
		break;
	case WM_COMMAND:
		if(!pThis->OnCommand(LOWORD(wParam), HIWORD(wParam), (HWND)lParam)) return FALSE;
		break;
	case WM_NOTIFY:
		if(!pThis->OnNotify(wParam, (NMHDR*)lParam)) return FALSE;
		break;
	case WM_TIMER:
		pThis->OnTimer();
		return FALSE;
		break;
	case WM_HSCROLL:
		if(!pThis->OnHScroll(LOWORD(wParam), HIWORD(wParam))) return FALSE;
		break;
	case WM_VSCROLL:
		if(!pThis->OnVScroll(LOWORD(wParam), HIWORD(wParam))) return FALSE;
		break;
	case WM_MOUSEMOVE:
		if(!pThis->OnMouseMove(wParam, LOWORD(lParam), HIWORD(lParam))) return FALSE;
		break;
	case WM_LBUTTONDOWN:
		if(!pThis->OnLeftButtonDown(LOWORD(lParam), HIWORD(lParam))) return FALSE;
		break;
	case WM_LBUTTONUP:
		if(!pThis->OnLeftButtonUp(LOWORD(lParam), HIWORD(lParam))) return FALSE;
		break;
	case WM_LBUTTONDBLCLK:
		if(!pThis->OnLeftButtonDblClk(LOWORD(lParam), HIWORD(lParam))) return FALSE;
		break;
	case WM_RBUTTONUP:
		if(!pThis->OnRightButtonUp(LOWORD(lParam), HIWORD(lParam))) return FALSE;
		break;
	case 0x20A:
		if(!pThis->OnMouseWheel((short)HIWORD(wParam))) return FALSE;
		break;
	case WM_MOUSELEAVE:
		if(!pThis->OnMouseLeave()) return FALSE;
		break;
	case WM_DESTROY:
		if(!pThis->OnDestroy()) return FALSE;
		break;
	case WM_CTLCOLORSTATIC:
		long nBrush;
		nBrush = pThis->OnCtlColorStatic((HDC)wParam, (HWND)lParam);
		if(!nBrush)
		{
			return (long)DefWindowProc(hWnd, uiMsg, wParam, lParam);
		}
		else
		{
			return nBrush;
		}
		break;
	case WM_ACTIVATE:
		if(!pThis->OnActivate(LOWORD(wParam), HIWORD(wParam) != 0, (HWND)lParam)) return FALSE;
		break;
	}
	if(!pThis->OnWndProc(uiMsg, wParam, lParam)) return FALSE;
	if(!pThis->m_nNoCallDef)
	{
		return (long)DefWindowProc(hWnd, uiMsg, wParam, lParam);
	}
	return TRUE;
}

long CWindow::OnWndProc(unsigned int uiMsg, WPARAM wParam, LPARAM lParam)
{
	return TRUE;
}

long CWindow::OnCommand(unsigned short nID, unsigned short nMsg, HWND hWnd)
{
	return TRUE;
}

long CWindow::OnSysCommand(unsigned int nCmd, LPARAM lParam)
{
	return TRUE;
}

long CWindow::OnNotify(WPARAM wParam, NMHDR* pHDR)
{
	return TRUE;
}

long CWindow::OnSize(unsigned int nType, unsigned int nWidth, unsigned int nHeight)
{
	return TRUE;
}

long CWindow::OnMouseLeave()
{
	return TRUE;
}

long CWindow::OnMouseMove(WPARAM nButton, unsigned int nX, unsigned int nY)
{
	return TRUE;
}

long CWindow::OnLeftButtonDown(unsigned int nX, unsigned int nY)
{
	return TRUE;
}

long CWindow::OnLeftButtonUp(unsigned int nX, unsigned int nY)
{
	return TRUE;
}

long CWindow::OnLeftButtonDblClk(unsigned int nX, unsigned int nY)
{
	return TRUE;
}

long CWindow::OnRightButtonUp(unsigned int nX, unsigned int nY)
{
	return TRUE;
}

long CWindow::OnMouseWheel(short nZ)
{
	return TRUE;
}

long CWindow::OnDestroy()
{
	return TRUE;
}

long CWindow::OnPaint()
{
	return TRUE;
}

long CWindow::OnTimer()
{
	return TRUE;
}

long CWindow::OnEraseBkgnd()
{
	return TRUE;
}

long CWindow::OnVScroll(unsigned int nType, unsigned int nTrackPos)
{
	return TRUE;
}

long CWindow::OnHScroll(unsigned int nType, unsigned int nTrackPos)
{
	return TRUE;
}

long CWindow::OnSysKeyDown(unsigned int nKey)
{
	return TRUE;
}

long CWindow::OnKeyDown(unsigned int nKey)
{
	return TRUE;
}

long CWindow::OnKeyUp(unsigned int nKey)
{
	return TRUE;
}

long CWindow::OnKillFocus()
{
	return TRUE;
}

long CWindow::OnSetFocus()
{
	return TRUE;
}

long CWindow::OnCtlColorStatic(HDC hDC, HWND hWnd)
{
	return FALSE;
}

long CWindow::OnActivate(unsigned int nType, bool bMinimized, HWND hWnd)
{
	return TRUE;
}
